@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020-2021 Salvador E. Tropea
-# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2022 Salvador E. Tropea
+# Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # Copyright (c) 2018 John Beard
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
@@ -14,8 +14,8 @@ from sys import (exit, maxsize)
 from collections import OrderedDict
 
 from .error import (KiPlotConfigurationError, config_error)
-from .kiplot import (load_board)
-from .misc import (NO_YAML_MODULE, EXIT_BAD_ARGS, EXAMPLE_CFG, WONT_OVERWRITE, W_NOOUTPUTS)
+from .misc import (NO_YAML_MODULE, EXIT_BAD_ARGS, EXAMPLE_CFG, WONT_OVERWRITE, W_NOOUTPUTS, W_UNKOUT, W_NOFILTERS,
+                   W_NOVARIANTS, W_NOGLOBALS)
 from .gs import GS
 from .registrable import RegOutput, RegVariant, RegFilter
 from .pre_base import BasePreFlight
@@ -23,7 +23,7 @@ from .pre_base import BasePreFlight
 # Logger
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 
 try:
     import yaml
@@ -36,6 +36,8 @@ except ImportError:
 class CfgYamlReader(object):
     def __init__(self):
         super().__init__()
+        self.imported_globals = {}
+        self.no_run_by_default = []
 
     def _check_version(self, v):
         if not isinstance(v, dict):
@@ -83,7 +85,22 @@ class CfgYamlReader(object):
         o_out.name = name
         o_out.type = otype
         o_out.comment = comment
-
+        o_out.extends = o_tree.get('extends', '')
+        # Pre-parse the run_by_default option
+        o_out.run_by_default = o_tree.get('run_by_default', True)
+        if not isinstance(o_out.run_by_default, bool):
+            o_out.run_by_default = True
+        o_out.disable_run_by_default = o_tree.get('disable_run_by_default', '')
+        # Pre-parse the disable_run_by_default option
+        if isinstance(o_out.disable_run_by_default, str):
+            if o_out.disable_run_by_default:
+                self.no_run_by_default.append(o_out.disable_run_by_default)
+        elif isinstance(o_out.disable_run_by_default, bool):
+            # True means to disable the one we extend
+            if o_out.disable_run_by_default and o_out.extends:
+                self.no_run_by_default.append(o_out.extends)
+        else:
+            o_out.disable_run_by_default = ''
         return o_out
 
     def _parse_outputs(self, v):
@@ -163,14 +180,127 @@ class CfgYamlReader(object):
         logger.debug("Parsing global options: {}".format(gb))
         if not isinstance(gb, dict):
             config_error("Incorrect `global` section (must be a dict)")
+        if self.imported_globals:
+            gb.update(self.imported_globals)
+            logger.debug("Global options + imported: {}".format(gb))
         # Parse all keys inside it
-        glb = GS.global_opts_class()
+        glb = GS.class_for_global_opts()
         glb.set_tree(gb)
         try:
             glb.config(None)
         except KiPlotConfigurationError as e:
             config_error("In `global` section: "+str(e))
 
+    @staticmethod
+    def _config_error_import(fname, error):
+        if fname is None:
+            fname = '*unnamed*'
+        config_error('{} in {} import'.format(error, fname))
+
+    @staticmethod
+    def _parse_import_items(kind, fname, value):
+        if isinstance(value, str):
+            if value == 'all':
+                return None
+            elif value == 'none':
+                return []
+            return [value]
+        if isinstance(value, list):
+            values = []
+            for v in value:
+                if isinstance(v, str):
+                    values.append(v)
+                else:
+                    CfgYamlReader._config_error_import(fname, '`{}` items must be strings ({})'.format(kind, str(v)))
+            return values
+        CfgYamlReader._config_error_import(fname, '`{}` must be a string or a list ({})'.format(kind, str(v)))
+
+    def _parse_import_outputs(self, outs, explicit_outs, fn_rel, data):
+        if (outs is None or len(outs) > 0) and 'outputs' in data:
+            i_outs = self._parse_outputs(data['outputs'])
+            if outs is not None:
+                sel_outs = []
+                for o in i_outs:
+                    if o.name in outs:
+                        sel_outs.append(o)
+                        outs.remove(o)
+                for o in outs:
+                    logger.warning(W_UNKOUT+"can't import `{}` output from `{}` (missing)".format(o, fn_rel))
+            else:
+                sel_outs = i_outs
+            if len(sel_outs) == 0:
+                logger.warning(W_NOOUTPUTS+"No outputs found in `{}`".format(fn_rel))
+            else:
+                try:
+                    RegOutput.add_outputs(sel_outs, fn_rel)
+                except KiPlotConfigurationError as e:
+                    config_error(str(e))
+                logger.debug('Outputs loaded from `{}`: {}'.format(fn_rel, list(map(lambda c: c.name, sel_outs))))
+        if outs is None and explicit_outs and 'outputs' not in data:
+            logger.warning(W_NOOUTPUTS+"No outputs found in `{}`".format(fn_rel))
+
+    def _parse_import_filters(self, filters, explicit_fils, fn_rel, data):
+        if (filters is None or len(filters) > 0) and 'filters' in data:
+            i_fils = self._parse_filters(data['filters'])
+            if filters is not None:
+                sel_fils = {}
+                for f in filters:
+                    if f in i_fils:
+                        sel_fils[f] = i_fils[f]
+                    else:
+                        logger.warning(W_UNKOUT+"can't import `{}` filter from `{}` (missing)".format(f, fn_rel))
+            else:
+                sel_fils = i_fils
+            if len(sel_fils) == 0:
+                logger.warning(W_NOFILTERS+"No filters found in `{}`".format(fn_rel))
+            else:
+                RegOutput.add_filters(sel_fils)
+                logger.debug('Filters loaded from `{}`: {}'.format(fn_rel, sel_fils.keys()))
+        if filters is None and explicit_fils and 'filters' not in data:
+            logger.warning(W_NOFILTERS+"No filters found in `{}`".format(fn_rel))
+
+    def _parse_import_variants(self, vars, explicit_vars, fn_rel, data):
+        if (vars is None or len(vars) > 0) and 'variants' in data:
+            i_vars = self._parse_variants(data['variants'])
+            if vars is not None:
+                sel_vars = {}
+                for f in vars:
+                    if f in i_vars:
+                        sel_vars[f] = i_vars[f]
+                    else:
+                        logger.warning(W_UNKOUT+"can't import `{}` variant from `{}` (missing)".format(f, fn_rel))
+            else:
+                sel_vars = i_vars
+            if len(sel_vars) == 0:
+                logger.warning(W_NOVARIANTS+"No variants found in `{}`".format(fn_rel))
+            else:
+                RegOutput.add_variants(sel_vars)
+                logger.debug('Variants loaded from `{}`: {}'.format(fn_rel, sel_vars.keys()))
+        if vars is None and explicit_vars and 'variants' not in data:
+            logger.warning(W_NOVARIANTS+"No variants found in `{}`".format(fn_rel))
+
+    def _parse_import_globals(self, globals, explicit_globals, fn_rel, data):
+        if (globals is None or len(globals) > 0) and 'global' in data:
+            i_globals = data['global']
+            if not isinstance(i_globals, dict):
+                config_error("Incorrect `global` section (must be a dict), while importing from {}".format(fn_rel))
+            if globals is not None:
+                sel_globals = {}
+                for f in globals:
+                    if f in i_globals:
+                        sel_globals[f] = i_globals[f]
+                    else:
+                        logger.warning(W_UNKOUT+"can't import `{}` global from `{}` (missing)".format(f, fn_rel))
+            else:
+                sel_globals = i_globals
+            if len(sel_globals) == 0:
+                logger.warning(W_NOGLOBALS+"No globals found in `{}`".format(fn_rel))
+            else:
+                self.imported_globals.update(sel_globals)
+                logger.debug('Globals loaded from `{}`: {}'.format(fn_rel, sel_globals.keys()))
+        if globals is None and explicit_globals and 'global' not in data:
+            logger.warning(W_NOGLOBALS+"No globals found in `{}`".format(fn_rel))
+
     def _parse_import(self, imp, name):
         """ Get imports """
         logger.debug("Parsing imports: {}".format(imp))
@@ -178,22 +308,57 @@ class CfgYamlReader(object):
             config_error("Incorrect `import` section (must be a list)")
         # Import the files
         dir = os.path.dirname(os.path.abspath(name))
-        outputs = []
-        for fn in imp:
-            if not isinstance(fn, str):
-                config_error("`import` items must be strings ({})".format(str(fn)))
+        for entry in imp:
+            if isinstance(entry, str):
+                fn = entry
+                outs = None
+                filters = []
+                vars = []
+                globals = []
+                explicit_outs = True
+                explicit_fils = False
+                explicit_vars = False
+                explicit_globals = False
+            elif isinstance(entry, dict):
+                fn = outs = filters = vars = globals = None
+                explicit_outs = explicit_fils = explicit_vars = explicit_globals = False
+                for k, v in entry.items():
+                    if k == 'file':
+                        if not isinstance(v, str):
+                            config_error("`import.file` must be a string ({})".format(str(v)))
+                        fn = v
+                    elif k == 'outputs':
+                        outs = self._parse_import_items('outputs', fn, v)
+                        explicit_outs = True
+                    elif k == 'filters':
+                        filters = self._parse_import_items('filters', fn, v)
+                        explicit_fils = True
+                    elif k == 'variants':
+                        vars = self._parse_import_items('variants', fn, v)
+                        explicit_vars = True
+                    elif k == 'global':
+                        globals = self._parse_import_items('global', fn, v)
+                        explicit_globals = True
+                    else:
+                        self._config_error_import(fn, "unknown import entry `{}`".format(str(v)))
+                if fn is None:
+                    config_error("`import` entry without `file` ({})".format(str(entry)))
+            else:
+                config_error("`import` items must be strings or dicts ({})".format(str(entry)))
             if not os.path.isabs(fn):
                 fn = os.path.join(dir, fn)
             if not os.path.isfile(fn):
                 config_error("missing import file `{}`".format(fn))
+            fn_rel = os.path.relpath(fn)
             data = self.load_yaml(open(fn))
-            if 'outputs' in data:
-                outs = self._parse_outputs(data['outputs'])
-                outputs.extend(outs)
-                logger.debug('Outputs loaded from `{}`: {}'.format(os.path.relpath(fn), list(map(lambda c: c.name, outs))))
-            else:
-                logger.warning(W_NOOUTPUTS+"No outputs found in `{}`".format(fn))
-        return outputs
+            # Outputs
+            self._parse_import_outputs(outs, explicit_outs, fn_rel, data)
+            # Filters
+            self._parse_import_filters(filters, explicit_fils, fn_rel, data)
+            # Variants
+            self._parse_import_variants(vars, explicit_vars, fn_rel, data)
+            # Globals
+            self._parse_import_globals(globals, explicit_globals, fn_rel, data)
 
     def load_yaml(self, fstream):
         try:
@@ -209,15 +374,10 @@ class CfgYamlReader(object):
         :param fstream: file stream of a config YAML file
         """
         data = self.load_yaml(fstream)
-        # Transfer command line global overwrites
-        GS.global_output = GS.global_from_cli.get('output', None)
-        GS.global_variant = GS.global_from_cli.get('variant', None)
-        GS.global_kiauto_wait_start = GS.global_from_cli.get('kiauto_wait_start', None)
-        GS.global_kiauto_time_out_scale = GS.global_from_cli.get('kiauto_time_out_scale', None)
         # List of outputs
-        outputs = []
         version = None
-        # Analize each section
+        globals_found = False
+        # Analyze each section
         for k, v in data.items():
             # logger.debug('{} {}'.format(k, v))
             if k == 'kiplot' or k == 'kibot':
@@ -226,19 +386,39 @@ class CfgYamlReader(object):
                 self._parse_preflight(v)
             elif k == 'global':
                 self._parse_global(v)
+                globals_found = True
             elif k == 'import':
-                outputs.extend(self._parse_import(v, fstream.name))
+                self._parse_import(v, fstream.name)
             elif k == 'variants':
-                RegOutput.set_variants(self._parse_variants(v))
+                RegOutput.add_variants(self._parse_variants(v))
             elif k == 'filters':
-                RegOutput.set_filters(self._parse_filters(v))
+                RegOutput.add_filters(self._parse_filters(v))
             elif k == 'outputs':
-                outputs.extend(self._parse_outputs(v))
+                try:
+                    RegOutput.add_outputs(self._parse_outputs(v))
+                except KiPlotConfigurationError as e:
+                    config_error(str(e))
             else:
                 config_error('Unknown section `{}` in config.'.format(k))
         if version is None:
             config_error("YAML config needs `kibot.version`.")
-        return outputs
+        # If no globals defined initialize them with default values
+        if not globals_found:
+            self._parse_global({})
+        # Solve the global variant
+        if GS.global_variant:
+            try:
+                GS.solved_global_variant = RegOutput.check_variant(GS.global_variant)
+            except KiPlotConfigurationError as e:
+                config_error("In global section: "+str(e))
+        # Ok, now we have all the outputs loaded, so we can apply the disable_run_by_default
+        for name in self.no_run_by_default:
+            o = RegOutput.get_output(name)
+            if o:
+                o.run_by_default = False
+                logger.debug("Disabling the default run for `{}`".format(o))
+
+        return RegOutput.get_outputs()
 
 
 def trim(docstring):
@@ -287,6 +467,7 @@ def print_output_options(name, cl, indent):
             entry = '  - `{}`: '
         if help is None:
             help = 'Undocumented'
+            logger.error('Undocumented option: `{}`'.format(k))
         lines = help.split('\n')
         preface = ind_str+entry.format(k)
         clines = len(lines)
@@ -294,7 +475,7 @@ def print_output_options(name, cl, indent):
         ind_help = len(preface)*' '
         for ln in range(1, clines):
             text = lines[ln].strip()
-            # Dots at the beggining are replaced by spaces.
+            # Dots at the beginning are replaced by spaces.
             # Used to keep indentation.
             if text[0] == '.':
                 for i in range(1, len(text)):
@@ -342,23 +523,23 @@ def print_output_help(name):
 
 
 def print_preflights_help():
-    pres = BasePreFlight.get_registered()
-    logger.debug('{} supported preflights'.format(len(pres)))
+    prefs = BasePreFlight.get_registered()
+    logger.debug('{} supported preflights'.format(len(prefs)))
     print('Supported preflight options:\n')
-    for n, o in OrderedDict(sorted(pres.items())).items():
+    for n, o in OrderedDict(sorted(prefs.items())).items():
         help, options = o.get_doc()
         if help is None:
             help = 'Undocumented'
-        print('- {}: {}.'.format(n, help.strip()))
+        print('- `{}`: {}.'.format(n, help.strip()))
         if options:
             print_output_options(n, options, 2)
 
 
 def print_filters_help():
-    fils = RegFilter.get_registered()
-    logger.debug('{} supported filters'.format(len(fils)))
+    filters = RegFilter.get_registered()
+    logger.debug('{} supported filters'.format(len(filters)))
     print('Supported filters:\n')
-    for n, o in OrderedDict(sorted(fils.items())).items():
+    for n, o in OrderedDict(sorted(filters.items())).items():
         help = o.__doc__
         if help is None:
             help = 'Undocumented'
@@ -366,13 +547,23 @@ def print_filters_help():
         print_output_options(n, o, 2)
 
 
+def print_global_options_help():
+    print_output_options('Global options', GS.class_for_global_opts, 2)
+
+
+def quoted(val):
+    if "'" in val:
+        return '"{}"'.format(val)
+    return "'{}'".format(val)
+
+
 def print_example_options(f, cls, name, indent, po, is_list=False):
     ind_str = indent*' '
     obj = cls()
     first = True
     if po:
         obj.read_vals_from_po(po)
-    for k, v in obj.get_attrs_gen():
+    for k, _ in obj.get_attrs_gen():
         help, alias, is_alias = obj.get_doc(k)
         if is_alias:
             f.write(ind_str+'# `{}` is an alias for `{}`\n'.format(k, alias))
@@ -380,7 +571,7 @@ def print_example_options(f, cls, name, indent, po, is_list=False):
         if help:
             help_lines = help.split('\n')
             for hl in help_lines:
-                # Dots at the beggining are replaced by spaces.
+                # Dots at the beginning are replaced by spaces.
                 # Used to keep indentation.
                 hl = hl.strip()
                 if hl[0] == '.':
@@ -395,7 +586,7 @@ def print_example_options(f, cls, name, indent, po, is_list=False):
         else:
             val = getattr(obj, k)
         if isinstance(val, str):
-            val = "'{}'".format(val)
+            val = quoted(val)
         elif isinstance(val, bool):
             val = str(val).lower()
         if isinstance(val, type):
@@ -405,14 +596,22 @@ def print_example_options(f, cls, name, indent, po, is_list=False):
                 # Get the text before the space, without the ]
                 txt = txt.split()[0][:-1]
                 f.write(ind_str+'{}: {}\n'.format(k, txt))
-            elif hasattr(val, '_default'):
-                f.write(ind_str+'{}: {}\n'.format(k, val._default))
+            elif val.get_default():
+                f.write(ind_str+'{}: {}\n'.format(k, val.get_default()))
             else:
+                if is_list and first:
+                    k = '- '+k
                 f.write(ind_str+'{}:\n'.format(k))
-                print_example_options(f, val, '', indent+2, None, help and 'list(dict' in help_lines[0])
+                extra_indent = 2 if not is_list else 4
+                print_example_options(f, val, '', indent+extra_indent, None, help and 'list(dict' in help_lines[0])
+                if is_list and first:
+                    ind_str += '  '
+                    first = False
         else:
             if is_list and first:
                 k = '- '+k
+            if val is None:
+                val = 'null'
             f.write(ind_str+'{}: {}\n'.format(k, val))
             if is_list and first:
                 ind_str += '  '
@@ -436,13 +635,16 @@ def create_example(pcb_file, out_dir, copy_options, copy_expand):
         f.write('kibot:\n  version: 1\n')
         # Preflights
         f.write('\npreflight:\n')
-        pres = BasePreFlight.get_registered()
-        for n, o in OrderedDict(sorted(pres.items())).items():
+        prefs = BasePreFlight.get_registered()
+        for n, o in OrderedDict(sorted(prefs.items())).items():
             if o.__doc__:
                 lines = trim(o.__doc__.rstrip()+'.')
                 for ln in lines:
                     f.write('  # '+ln.rstrip()+'\n')
-            f.write('  {}: {}\n'.format(n, o.get_example()))
+            example = o.get_example()
+            if not example.startswith("\n"):
+                example = ' '+example
+            f.write('  {}:{}\n'.format(n, example))
         # Outputs
         outs = RegOutput.get_registered()
         f.write('\noutputs:\n')
@@ -451,7 +653,7 @@ def create_example(pcb_file, out_dir, copy_options, copy_expand):
         layers = 'all'
         if pcb_file:
             # We have a PCB to take as reference
-            board = load_board(pcb_file)
+            board = GS.load_board(pcb_file)
             if copy_options or copy_expand:
                 # Layers and plot options from the PCB
                 layers = 'selected'
@@ -464,7 +666,7 @@ def create_example(pcb_file, out_dir, copy_options, copy_expand):
             for ln in range(2, len(lines)):
                 f.write('  # '+lines[ln].rstrip()+'\n')
             f.write("  - name: '{}_example'\n".format(n))
-            f.write("    comment: '{}'\n".format(lines[1]))
+            f.write("    comment: {}\n".format(quoted(lines[1])))
             f.write("    type: '{}'\n".format(n))
             f.write("    dir: 'Example/{}_dir'\n".format(n))
             f.write("    options:\n")
@@ -481,4 +683,3 @@ def create_example(pcb_file, out_dir, copy_options, copy_expand):
                             f.write("        description: '{}'\n".format(layer.description))
                 else:
                     f.write('    layers: {}\n'.format(layers))
-            f.write('\n')
