@@ -10,7 +10,7 @@ from .gs import GS
 from .macros import macros, document, filter_class  # noqa: F401
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 
 
 @filter_class
@@ -26,7 +26,7 @@ class Var_Rename(BaseFilter):  # noqa: F821
             self.variant_to_value = False
             """ Rename fields matching the variant to the value of the component """
             self.force_variant = ''
-            """ Use this variant instead of the current variant. Usefull for IBoM variants """
+            """ Use this variant instead of the current variant. Useful for IBoM variants """
 
     def config(self, parent):
         super().config(parent)
