@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020-2021 Salvador E. Tropea
-# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2022 Salvador E. Tropea
+# Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """
@@ -11,6 +11,8 @@ Currently oriented to collect the components for the BoM.
 # Encapsulate file/line
 import re
 import os
+from xml.etree.ElementTree import Element, SubElement, tostring
+from xml.dom import minidom
 from datetime import datetime
 from copy import deepcopy
 from collections import OrderedDict
@@ -18,10 +20,10 @@ from .config import KiConf, un_quote
 from ..gs import GS
 from ..misc import (W_BADPOLI, W_POLICOORDS, W_BADSQUARE, W_BADCIRCLE, W_BADARC, W_BADTEXT, W_BADPIN, W_BADCOMP, W_BADDRAW,
                     W_UNKDCM, W_UNKAR, W_ARNOPATH, W_ARNOREF, W_MISCFLD, W_EXTRASPC, W_NOLIB, W_INCPOS, W_NOANNO, W_MISSLIB,
-                    W_MISSDCM, W_MISSCMP, W_MISFLDNAME)
+                    W_MISSDCM, W_MISSCMP, W_MISFLDNAME, W_NOENDLIB)
 from .. import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 
 
 class SchError(Exception):
@@ -83,13 +85,28 @@ class DCMLineReader(LineReader):
     def __init__(self, f, file):
         super().__init__(f, file)
 
-    def get_line(self):
+    def readline(self):
         res = self.f.readline()
+        try:
+            res = res.decode()
+        except UnicodeDecodeError:
+            logger.error('Invalid UTF-8 sequence at line {} of file `{}`'.format(self.line+1, self.file))
+            nres = ''
+            for c in res:
+                if c > 127:
+                    c = 32
+                nres += chr(c)
+            res = nres
+            logger.error('Using: '+res.rstrip())
+        return res
+
+    def get_line(self):
+        res = self.readline()
         while res and res[0] == '#':
             if res.startswith('#End Doc Library'):
                 return res.rstrip()
             self.line += 1
-            res = self.f.readline()
+            res = self.readline()
         if not res:
             raise SchLibError('Unexpected end of file', '', self)
         self.line += 1
@@ -105,16 +122,28 @@ class LibComponentField(object):
     """ A field for a component in the library.
         Almost the same as a field in the schematic, but incompatible!!! """
     # F n "text" posx posy dimension orientation visibility hjustify vjustify/italic/bold "name"
-    field_re = re.compile(r'F\s*(\d+)\s+'  # 0 Field number
-                          r'"([^"]*)"\s+'  # 1 Field value
-                          r'(-?\d+)\s+'    # 2 Pos X
-                          r'(-?\d+)\s+'    # 3 Pos Y
-                          r'(\d+)\s+'      # 4 Dimension
-                          r'([HV])\s+'     # 5 Orientation
-                          r'([VI])\s+'     # 6 Visibility
-                          r'([LRCBT])\s+'  # 7 HJustify
-                          r'([LRCBT][IN][BN])\s*'  # 8 VJustify+Italic+Bold
-                          r'("[^"]*")?')   # 9 Name for user fields
+    field_re = re.compile(r'F\s*(\d+)\s+'               # 0 Field number
+                          r'"((?:[^\\]|(?:\\.))*)"\s+'  # 1 Field value
+                          r'(-?\d+)\s+'                 # 2 Pos X
+                          r'(-?\d+)\s+'                 # 3 Pos Y
+                          r'(\d+)\s+'                   # 4 Dimension
+                          r'([HV])\s+'                  # 5 Orientation
+                          r'([VI])\s+'                  # 6 Visibility
+                          r'([LRCBT])\s+'               # 7 HJustify
+                          r'([LRCBT][IN][BN])\s*'       # 8 VJustify+Italic+Bold
+                          r'("(?:[^\\]|(?:\\.))*")?')   # 9 Name for user fields
+    fiel2_re = re.compile(r'F\s*(\d+)\s+'               # 0 Field number
+                          r'"((?:[^\\]|(?:\\.))*)"\s+'  # 1 Field value
+                          r'(-?\d+)\s+'                 # 2 Pos X
+                          r'(-?\d+)\s+'                 # 3 Pos Y
+                          r'(\d+)\s+'                   # 4 Dimension
+                          r'([HV])\s+'                  # 5 Orientation
+                          r'([VI])\s+'                  # 6 Visibility
+                          r'([LRCBT])\s+'               # 7 HJustify
+                          # KiCad never uses spaces between "CNN", but can load files with it
+                          # Some generators seems to use it see #122
+                          r'([LRCBT]\s*[IN]\s*[BN])\s*'  # 8 VJustify+Italic+Bold
+                          r'("(?:[^\\]|(?:\\.))*")?')    # 9 Name for user fields
 
     def __init__(self):
         super().__init__()
@@ -123,7 +152,9 @@ class LibComponentField(object):
     def parse(line, lib_name, f):
         m = LibComponentField.field_re.match(line)
         if not m:
-            raise SchLibError('Malformed component field', line, f)
+            m = LibComponentField.fiel2_re.match(line)
+            if not m:
+                raise SchLibError('Malformed component field', line, f)
         field = LibComponentField()
         gs = m.groups()
         field.number = int(gs[0])
@@ -166,7 +197,7 @@ class DrawPoligon(object):
     pol_re = re.compile(r'P\s+(\d+)\s+'     # 0 Number of points
                         r'(\d+)\s+'         # 1 Sub-part (0 == all)
                         r'([012])\s+'       # 2 Which representation (0 == both) for DeMorgan
-                        r'(-?\d+)\s+'       # 3 Thickness (Components from 74xx.lib has poligons with -1000)
+                        r'(-?\d+)\s+'       # 3 Thickness (Components from 74xx.lib has polygons with -1000)
                         r'((?:-?\d+\s+)+)'  # 4 The points
                         r'([NFf])')         # 5 Normal, Filled
 
@@ -177,7 +208,7 @@ class DrawPoligon(object):
     def parse(line):
         m = DrawPoligon.pol_re.match(line)
         if not m:
-            logger.warning(W_BADPOLI + 'Unknown poligon definition `{}`'.format(line))
+            logger.warning(W_BADPOLI + 'Unknown polygon definition `{}`'.format(line))
             return None
         pol = DrawPoligon()
         g = m.groups()
@@ -188,7 +219,7 @@ class DrawPoligon(object):
         pol.fill = g[5]
         coords = _split_space(g[4])
         if len(coords) != 2*pol.points:
-            logger.warning(W_POLICOORDS + 'Expected {} coordinates and got {} in poligon'.format(2*pol.points, len(coords)))
+            logger.warning(W_POLICOORDS + 'Expected {} coordinates and got {} in polygon'.format(2*pol.points, len(coords)))
             pol.points = int(len(coords)/2)
         pol.coords = [int(c) for c in coords]
         return pol
@@ -419,6 +450,8 @@ class Pin(object):
                         r'([012])\s+'       # 9 Which representation (0 == both) for DeMorgan
                         r'([IOBTPUWwCEN])'  # 10 Electrical type
                         r'((?:\s+)\S+)?')   # 11 Graphic type
+    type2name = {'I': 'input', 'O': 'output', 'B': 'BiDi', 'T': '3state', 'P': 'passive', 'U': 'unspc',
+                 'W': 'power_in', 'w': 'power_out', 'C': 'openCol', 'E': 'openEm', 'N': 'NotConnected'}
 
     def __init__(self):
         super().__init__()
@@ -508,8 +541,13 @@ class LibComponent(object):
         self.alias = None
         self.fp_list = []
         self.draw = []
+        self.pins = []
         line = f.get_line()
         while not line.startswith('ENDDEF'):
+            if len(line) == 0:
+                # Skip empty lines
+                line = f.get_line()
+                continue
             if line[0] == 'F':
                 # A field
                 field = LibComponentField.parse(line, lib_name, f)
@@ -536,11 +574,20 @@ class LibComponent(object):
                     elif line[0] == 'T':
                         self.draw.append(DrawText.parse(line))
                     elif line[0] == 'X':
-                        self.draw.append(Pin.parse(line))
+                        pin = Pin.parse(line)
+                        self.draw.append(pin)
+                        self.pins.append(pin)
                     else:
                         logger.warning(W_BADDRAW + 'Unknown draw element `{}`'.format(line))
                     line = f.get_line()
             line = f.get_line()
+            self.all_pins = self.pins
+
+    def get_field_value(self, field):
+        field = field.lower()
+        if field in self.dfields:
+            return self.dfields[field].value
+        return ''
 
     def write(self, f, id, cross=False):
         """ cross is used to cross the component (DNF) """
@@ -614,7 +661,13 @@ class SymLib(object):
         self.alias = {}
 
     @staticmethod
-    def _check_add(o, id, lib, needed):
+    def _check_add(o, id, lib, needed, translate):
+        if lib is None:
+            # From a cache
+            if id in translate:
+                needed[translate[id]] = o
+                return True
+            return False
         name = lib+':'+id
         if name in needed:
             needed[name] = o
@@ -635,20 +688,25 @@ class SymLib(object):
             if not line.startswith('EESchema-LIBRARY'):
                 raise SchLibError('Missing library signature', line, f)
             line = f.get_line()
+            translate = {k.replace(':', '_'): k for k, v in needed.items() if v is None} if lib_alias is None else None
             while not (line.startswith('#End Library') or line.startswith('# End Library')):
                 if line.startswith('DEF'):
                     o = LibComponent(line, f, file)
                     if o.name:
                         # Only add components we need
-                        if self._check_add(o, o.name, lib_alias, needed):
+                        if self._check_add(o, o.name, lib_alias, needed, translate):
                             self.comps[o.name] = o
-                        if o.alias:
+                        if o.alias and lib_alias is not None:
                             for a in o.alias:
-                                if self._check_add(o, a, lib_alias, needed):
+                                if self._check_add(o, a, lib_alias, needed, translate):
                                     self.alias[a] = o
                 else:
                     raise SchLibError('Unknown library entry', line, f)
-                line = f.get_line()
+                try:
+                    line = f.get_line()
+                except SchLibError:
+                    logger.warning(W_NOENDLIB + 'Library without end of file comment: `{}`'.format(file))
+                    break
 
 
 class DocLibEntry(object):
@@ -687,7 +745,7 @@ class DocLib(object):
     def load(self, file):
         """ Populates the class, file must exist """
         logger.debug('Loading doc-lib `{}`'.format(file))
-        with open(file, 'rt') as fh:
+        with open(file, 'rb') as fh:
             f = DCMLineReader(fh, file)
             line = f.get_line()
             if not line.startswith('EESchema-DOCLIB'):
@@ -706,11 +764,20 @@ class DocLib(object):
 
 class SchematicField(object):
     # F n "text" orientation posx posy dimension flags hjustify vjustify/italic/bold "name"
-    field_re = re.compile(r'F\s*(\d+)\s+"([^"]*)"\s+([HV])\s+(-?\d+)\s+(-?\d+)\s+(\d+)\s+(\d+)'
-                          r'\s+([LRCBT])\s+([LRCBT][IN][BN])\s*("[^"]*")?')
+    field_re = re.compile(r'F\s*(\d+)\s+"((?:[^\\]|(?:\\.))*)"\s+([HV])\s+(-?\d+)\s+(-?\d+)\s+(\d+)\s+(\d+)'
+                          r'\s+([LRCBT])\s+([LRCBT][IN][BN])\s*("(?:[^\\]|(?:\\.))*")?')
 
     def __init__(self):
         super().__init__()
+        self.horizontal = True  # H -> True, V -> False
+        self.x = 0
+        self.y = 0
+        self.size = 50
+        self.flags = '0001'
+        self.hjustify = 'C'
+        self.vjustify = 'C'
+        self.italic = False
+        self.bold = False
 
     @staticmethod
     def parse(line, f):
@@ -802,7 +869,11 @@ class SchematicComponent(object):
         - fixed: means you can't change it by a replacement without authorization
                  Is just a flag and doesn't affect much.
         - footprint_rot: angle to rotate the part in the pick & place.
-        - qty: ammount of this part used.
+        - footprint_x: x position of the part in the pick & place.
+        - footprint_y: y position of the part in the pick & place.
+        - footprint_w: width of the footprint (pads only).
+        - footprint_h: height of the footprint (pads only)
+        - qty: amount of this part used.
         """
     ref_re = re.compile(r'([^\d]+)([\?\d]+)')
 
@@ -823,7 +894,10 @@ class SchematicComponent(object):
         self.fixed = False
         self.bottom = False
         self.footprint_rot = 0.0
+        self.footprint_x = self.footprint_y = 0
+        self.footprint_w = self.footprint_h = 0
         self.qty = 1
+        self.annotation_error = False
         # KiCad 5 PCB flags (mutually exclusive)
         self.smd = False
         self.virtual = False
@@ -856,7 +930,7 @@ class SchematicComponent(object):
             if target.number < 4:
                 self._solve_fields(LineReader(None, '**Internal**'))
         else:
-            f = SchematicField()
+            f = type(self.fields[0])()
             f.name = field
             f.value = value
             f.number = self.get_free_field_number()
@@ -895,7 +969,7 @@ class SchematicComponent(object):
             self.dfields_bkp = {f.name.lower(): f for f in self.fields_bkp}
 
     def _solve_ref(self, path):
-        """ Look fo the correct reference for this path.
+        """ Look for the correct reference for this path.
             Returns the default reference if no paths defined.
             Returns the first not empty reference if the current is empty. """
         ref = self.f_ref
@@ -936,7 +1010,10 @@ class SchematicComponent(object):
                     self.footprint_lib = res[0]
                     self.footprint = res[1]
                 else:
-                    raise SchFileError('Footprint with more than one colon', f.value, fr)
+                    if fr:
+                        raise SchFileError('Footprint with more than one colon', f.value, fr)
+                    else:
+                        raise SchError('Footprint with more than one colon (`{}`)'.format(f.value))
                 basic += 1
             elif f.number == 3:
                 self.datasheet = f.value
@@ -1053,6 +1130,7 @@ class SchematicComponent(object):
         comp.is_power = comp.ref.startswith('#PWR') or comp.ref.startswith('#FLG')
         if comp.ref[-1] == '?':
             logger.warning(W_NOANNO + 'Component {} is not annotated'.format(comp))
+            comp.annotation_error = True
         # Separate the reference in its components
         m = SchematicComponent.ref_re.match(comp.ref)
         if not m:
@@ -1067,11 +1145,14 @@ class SchematicComponent(object):
         comp._validate()
         return comp
 
-    def write(self, f):
-        # Fake lib to reflect fitted status
-        lib = 'y' if self.fitted or not self.included else 'n'
-        # Fake name using cache style
-        name = '{}:{}_{}'.format(lib, self.lib, self.name)
+    def write(self, f, crossed=False):
+        if crossed:
+            # Fake lib to reflect fitted status
+            lib = 'y' if self.fitted or not self.included else 'n'
+            # Fake name using cache style
+            name = '{}:{}_{}'.format(lib, self.lib, self.name)
+        else:
+            name = '{}:{}'.format(self.lib, self.name)
         f.write('$Comp\n')
         f.write('L {} {}\n'.format(name, self.f_ref))
         f.write('U {} {} {}\n'.format(self.unit, self.unit2, self.id))
@@ -1277,7 +1358,7 @@ class SchematicBitmap(object):
     def write(self, f):
         f.write('$Bitmap\n')
         f.write('Pos {} {}\n'.format(self.x, self.y))
-        f.write('Scale {}\n'.format(self.scale))
+        f.write('Scale {0:.6f}\n'.format(self.scale))
         f.write('Data')
         for c, b in enumerate(self.data):
             if (c % 32) == 0:
@@ -1320,20 +1401,23 @@ class SchematicSheet(object):
         super().__init__()
         self.sheet = None
         self.id = ''
+        self.annotation_error = False
 
-    def load_sheet(self, project, parent, sheet_path, sheet_path_h, libs, fields, fields_lc):
+    def load_sheet(self, project, parent, sheet_path, sheet_path_h, libs, fields, fields_lc, parent_obj):
         assert self.name
         self.sheet = Schematic()
+        parent_obj.all_sheets.append(self.sheet)
         parent_dir = os.path.dirname(parent)
         sheet_path += '/'+self.id
         if len(sheet_path_h) > 1:
             sheet_path_h += '/'
         sheet_path_h += self.name if self.name else 'Unknown'
-        self.sheet.load(os.path.join(parent_dir, self.file), project, sheet_path, sheet_path_h, libs, fields, fields_lc)
+        self.sheet.load(os.path.join(parent_dir, self.file), project, sheet_path, sheet_path_h, libs, fields, fields_lc,
+                        parent_obj)
         return self.sheet
 
     @staticmethod
-    def load(f):
+    def load(f, parent):
         # Position & Size
         line = f.get_line()
         if line[0] != 'S':
@@ -1370,6 +1454,8 @@ class SchematicSheet(object):
                     raise SchFileError('Malformed sheet file name', line, f)
                 sch.file = m.group(1)
                 sch.file_size = int(m.group(2))
+                if not os.path.isfile(os.path.join(os.path.dirname(parent), sch.file)):
+                    raise SchFileError('Missing sub-sheet `{}`'.format(sch.file), line, f)
             else:
                 sch.labels.append(SchematicPort.parse(line[1:], f))
             line = f.get_line()
@@ -1392,30 +1478,47 @@ class SchematicSheet(object):
         f.write('$EndSheet\n')
 
 
+def _path(p):
+    if not p.startswith('/'):
+        p = '/'+p
+    if p[-1] != '/':
+        p = p+'/'
+    return p
+
+
 class Schematic(object):
     def __init__(self):
         super().__init__()
         self.dcms = {}
         self.lib_comps = {}
+        self.annotation_error = False
+        self.max_comments = 4
+        self.netlist_version = 'D'
 
     def _get_title_block(self, f):
         line = f.get_line()
-        m = re.match(r'\$Descr (\S+) (\d+) (\d+)', line)
+        m = re.match(r'\$Descr (\S+) (\d+) (\d+)( portrait)?', line)
         if not m:
             raise SchFileError('Missing $Descr', line, f)
         self.page_type = m.group(1)
         self.page_width = m.group(2)
         self.page_height = m.group(3)
+        self.paper_orientation = None
+        if m.group(4):
+            self.paper_orientation = m.group(4).strip()
         self.sheet = 1
         self.nsheets = 1
         self.title_block = OrderedDict()
+        self.comment = ['']*9
         while True:
             line = f.get_line()
             if line.startswith('$EndDescr'):
-                self.title = self.title_block['Title'] if 'Title' in self.title_block else ''
-                self.date = self.title_block['Date'] if 'Date' in self.title_block else ''
-                self.revision = self.title_block['Rev'] if 'Rev' in self.title_block else ''
-                self.company = self.title_block['Comp'] if 'Comp' in self.title_block else ''
+                self.title_ori = self.title = self.title_block.get('Title', '')
+                self.date = self.title_block.get('Date', '')
+                self.revision = self.title_block.get('Rev', '')
+                self.company = self.title_block.get('Comp', '')
+                for num in range(4):
+                    self.comment[num] = self.title_block.get('Comment'+str(num+1), '')
                 return
             elif line.startswith('encoding'):
                 if line[9:14] != 'utf-8':
@@ -1432,16 +1535,24 @@ class Schematic(object):
                     raise SchFileError('Wrong entry in title block', line, f)
                 self.title_block[m.group(1)] = m.group(2)
 
-    def load(self, fname, project, sheet_path='', sheet_path_h='/', libs={}, fields=[], fields_lc=set()):
+    def load(self, fname, project, sheet_path='', sheet_path_h='/', libs=None, fields=None, fields_lc=None, parent=None):
         """ Load a v5.x KiCad Schematic.
             The caller must be sure the file exists.
             Only the schematics are loaded not the libs. """
         logger.debug("Loading sheet from "+fname)
         self.fname = fname
+        if libs is None:
+            libs = {}
         self.libs = libs
+        if fields is None:
+            fields = []
         self.fields = fields
+        if fields_lc is None:
+            fields_lc = set()
         self.fields_lc = fields_lc
         self.project = project
+        self.sheet_path = sheet_path
+        self.sheet_path_h = sheet_path_h
         with open(fname, 'rt') as fh:
             f = SCHLineReader(fh, fname)
             line = f.get_line()
@@ -1464,9 +1575,7 @@ class Schematic(object):
             # Load the title block
             self._get_title_block(f)
             # Fill in some missing info
-            if not self.date:
-                file_mtime = os.path.getmtime(fname)
-                self.date = datetime.fromtimestamp(file_mtime).strftime('%Y-%m-%d_%H-%M-%S')
+            self.date = GS.format_date(self.date, fname, 'SCH')
             if not self.title:
                 self.title = os.path.splitext(os.path.basename(fname))[0]
             logger.debug("SCH title: `{}`".format(self.title))
@@ -1484,6 +1593,8 @@ class Schematic(object):
             while not line.startswith('$EndSCHEMATC'):
                 if line.startswith('$Comp'):
                     obj = SchematicComponent.load(f, project, sheet_path, sheet_path_h, libs, fields, fields_lc)
+                    if obj.annotation_error:
+                        self.annotation_error = True
                     self.components.append(obj)
                 elif line.startswith('NoConn'):
                     obj = SchematicConnection.parse(False, line[7:], f)
@@ -1501,7 +1612,7 @@ class Schematic(object):
                     obj = SchematicBitmap.load(f)
                     self.bitmaps.append(obj)
                 elif line.startswith('$Sheet'):
-                    obj = SchematicSheet.load(f)
+                    obj = SchematicSheet.load(f, fname)
                     self.sheets.append(obj)
                 else:
                     raise SchFileError('Unknown definition', line, f)
@@ -1509,8 +1620,14 @@ class Schematic(object):
                 line = f.get_line()
             # Load sub-sheets
             self.sub_sheets = []
+            if parent is None:
+                self.all_sheets = [self]
             for sch in self.sheets:
-                self.sub_sheets.append(sch.load_sheet(project, fname, sheet_path, sheet_path_h, libs, fields, fields_lc))
+                sheet = sch.load_sheet(project, fname, sheet_path, sheet_path_h, libs, fields, fields_lc,
+                                       self if parent is None else parent)
+                if sheet.annotation_error:
+                    self.annotation_error = True
+                self.sub_sheets.append(sheet)
 
     def get_files(self):
         """ A list of the names for all the sheets, including this one.
@@ -1525,7 +1642,7 @@ class Schematic(object):
         if exclude_power:
             components = [c for c in self.components if not c.is_power]
         else:
-            components = [c for c in self.components]
+            components = list(self.components)
         for sch in self.sheets:
             components.extend(sch.sheet.get_components(exclude_power))
         components.sort(key=lambda g: g.ref)
@@ -1598,6 +1715,13 @@ class Schematic(object):
                 # Mark as None if we don't know the file
                 self.lib_comps[k] = None
                 self.dcms[k] = None
+        # Do we have all the components?
+        if next((k for k, v in self.comps_data.items() if v is None), None) is not None:
+            cache_name = fname.replace('.sch', '-cache.lib')
+            if os.path.isfile(cache_name):
+                logger.debug("Loading missing components from cache "+cache_name)
+                o = SymLib()
+                o.load(cache_name, None, self.comps_data)
         if GS.debug_level > 1:
             logger.debug("Components after loading: "+str(self.comps_data))
         # Join the descriptions with the components
@@ -1609,6 +1733,11 @@ class Schematic(object):
                     comp.dcm = dcm.comps.get(name)
                     if not comp.dcm and k+':'+name in self.comps_data:
                         logger.warning(W_MISSDCM + 'Missing doc-lib entry for {}:{}'.format(k, name))
+                # Also do it for the aliases
+                for name, comp in lib.alias.items():
+                    comp.dcm = dcm.comps.get(name)
+                    if not comp.dcm and k+':'+name in self.comps_data:
+                        logger.warning(W_MISSDCM + 'Missing doc-lib entry for {}:{}'.format(k, name))
         # Transfer the descriptions to the instances of the components
         self.walk_components(self.apply_dcm, self)
 
@@ -1625,29 +1754,67 @@ class Schematic(object):
                     logger.warning(W_MISSCMP + 'Missing component `{}`'.format(k))
             f.write('#\n#End Library\n')
 
-    def save(self, fname, dest_dir):
-        fname = os.path.join(dest_dir, fname)
-        with open(fname, 'wt') as f:
-            f.write('EESchema Schematic File Version {}\n'.format(self.version))
-            f.write('EELAYER {} {}\n'.format(self.eelayer_n, self.eelayer_m))
-            f.write('EELAYER END\n')
-            f.write('$Descr {} {} {}\n'.format(self.page_type, self.page_width, self.page_height))
-            f.write('encoding utf-8\n')
-            f.write('Sheet {} {}\n'.format(self.sheet, self.nsheets))
-            for k, v in self.title_block.items():
-                f.write('{} "{}"\n'.format(k, v))
-            f.write('$EndDescr\n')
-            for e in self.all:
-                e.write(f)
-            f.write('$EndSCHEMATC\n')
+    def save(self, fname=None, dest_dir=None, base_sheet=None, saved=None):
+        """ Save the schematic and its sub-sheets.
+            If dest_dir is not None all files are stored in dest_dir (for variants). """
+        if base_sheet is None:
+            # We are the base sheet
+            base_sheet = self
+        if saved is None:
+            # Start memorizing saved files
+            saved = set()
+        if fname is None:
+            # Use our name if none provided
+            fname = self.fname
+        if dest_dir is None:
+            # Save at the same place
+            if not os.path.isabs(fname):
+                # Use the base sheet as reference
+                fname = os.path.join(os.path.dirname(base_sheet.fname), fname)
+        else:
+            # Save all in dest_dir (variant)
+            fname = os.path.join(dest_dir, fname)
+        # Save the sheet
+        if fname not in saved:
+            logger.debug('Saving schematic: `{}`'.format(fname))
+            # Keep a back-up of existing files
+            if os.path.isfile(fname):
+                bkp = fname+'-bak'
+                if os.path.isfile(bkp):
+                    os.remove(bkp)
+                os.rename(fname, bkp)
+            with open(fname, 'wt') as f:
+                f.write('EESchema Schematic File Version {}\n'.format(self.version))
+                f.write('EELAYER {} {}\n'.format(self.eelayer_n, self.eelayer_m))
+                f.write('EELAYER END\n')
+                f.write('$Descr {} {} {}'.format(self.page_type, self.page_width, self.page_height))
+                if self.paper_orientation:
+                    f.write(' {}'.format(self.paper_orientation))
+                f.write('\n')
+                f.write('encoding utf-8\n')
+                f.write('Sheet {} {}\n'.format(self.sheet, self.nsheets))
+                for k, v in self.title_block.items():
+                    f.write('{} "{}"\n'.format(k, v))
+                f.write('$EndDescr\n')
+                crossed = dest_dir is not None
+                for e in self.all:
+                    if isinstance(e, SchematicComponent):
+                        e.write(f, crossed)
+                    else:
+                        e.write(f)
+                f.write('$EndSCHEMATC\n')
+            saved.add(fname)
         # Save sub-sheets
         for c, sch in enumerate(self.sheets):
-            # Fake file name
-            file = sch.file.replace('/', '_')
-            self.sub_sheets[c].save(file, dest_dir)
+            file = sch.file
+            if dest_dir is not None:
+                # Fake file name, forcing a flat structure (all files in dest_dir)
+                file = file.replace('/', '_')
+            self.sub_sheets[c].save(file, dest_dir, base_sheet, saved)
 
     def save_variant(self, dest_dir):
-        # Currently imposible
+        """ Save a modified schematic with crossed components """
+        # Currently impossible
         # if not os.path.exists(dest_dir):
         #    os.makedirs(dest_dir)
         lib_yes = os.path.join(dest_dir, 'y.lib')
@@ -1675,3 +1842,189 @@ class Schematic(object):
             sch = os.path.basename(sch)
             fnames.append(os.path.join(dest_dir, sch.replace('/', '_')))
         return fnames
+
+    def save_netlist_design(self, root):
+        """ Generates the `design` section of the netlist """
+        design = SubElement(root, 'design')
+        SubElement(design, 'source').text = self.fname
+        SubElement(design, 'date').text = datetime.now().strftime("%c")
+        SubElement(design, 'tool').text = 'KiBot v'+GS.kibot_version
+        order = 1
+        is_v5 = self.max_comments == 4
+        for s in self.all_sheets:
+            sheet = SubElement(design, 'sheet')
+            # KiCad v5 numbering is broken
+            sheet.set('number', str(order) if is_v5 else s.sheet)
+            sheet.set('name', _path(s.sheet_path_h))
+            sheet.set('tstamps', _path(s.sheet_path))
+            tblock = SubElement(sheet, 'title_block')
+            title = SubElement(tblock, 'title')
+            if s.title_ori:
+                title.text = s.title_ori
+            company = SubElement(tblock, 'company')
+            if s.company:
+                company.text = s.company
+            rev = SubElement(tblock, 'rev')
+            if s.revision:
+                rev.text = s.revision
+            dt = SubElement(tblock, 'date')
+            if s.date:
+                dt.text = s.date
+            SubElement(tblock, 'source').text = os.path.basename(s.fname)
+            for num in range(s.max_comments):
+                com = SubElement(tblock, 'comment')
+                com.set('number', str(num+1))
+                com.set('value', s.comment[num])
+            order += 1
+
+    def save_netlist_components(self, root, comps, excluded, fitted, no_field):
+        """ Generates the `components` section of the netlist """
+        components = SubElement(root, 'components')
+        # Colapse units
+        real_comps = []
+        tstamps = {}
+        for c in comps:
+            if c.ref in tstamps:
+                tstamps[c.ref] += ' '+c.id
+            else:
+                tstamps[c.ref] = c.id
+                real_comps.append(c)
+        for c in real_comps:
+            if not excluded and not c.included:
+                # Excluded, i.e. virtual
+                continue
+            if fitted and not c.fitted:
+                # DNP
+                continue
+            comp = SubElement(components, 'comp')
+            comp.set('ref', c.ref)
+            SubElement(comp, 'value').text = c.value
+            if c.footprint:
+                SubElement(comp, 'footprint').text = c.footprint
+            if len(c.datasheet) and not (self.netlist_version == 'E' and c.datasheet != '~'):
+                SubElement(comp, 'datasheet').text = c.datasheet
+            user_fields = c.get_user_fields()
+            if user_fields:
+                fields = SubElement(comp, 'fields')
+                for fname, fvalue in user_fields:
+                    if fname in no_field:
+                        continue
+                    fld = SubElement(fields, 'field')
+                    fld.set('name', fname)
+                    fld.text = fvalue
+            lbs = SubElement(comp, 'libsource')
+            lbs.set('lib', c.lib)
+            lbs.set('part', c.name)
+            lbs.set('description', c.desc)
+            # v6 properties
+            if self.netlist_version == 'E':
+                for fname, fvalue in user_fields:
+                    if fname in no_field:
+                        continue
+                    fld = SubElement(comp, 'property')
+                    fld.set('name', fname)
+                    fld.set('value', fvalue)
+                prop = SubElement(comp, 'property')
+                prop.set('name', 'Sheetname')
+                prop.set('value', os.path.basename(c.sheet_path_h))
+                prop = SubElement(comp, 'property')
+                prop.set('name', 'Sheetfile')
+                prop.set('value', os.path.basename(c.parent_sheet.fname))
+            shp = SubElement(comp, 'sheetpath')
+            shp.set('names', _path(c.sheet_path_h))
+            shp.set('tstamps', _path(c.sheet_path))
+            if self.netlist_version == 'D':
+                SubElement(comp, 'tstamp').text = tstamps[c.ref].split()[0]
+            else:
+                SubElement(comp, 'tstamps').text = tstamps[c.ref]
+
+    def save_netlist_libparts(self, root):
+        libparts = SubElement(root, 'libparts')
+        for k in sorted(self.comps_data.keys()):
+            v = self.comps_data[k]
+            if not v:
+                continue
+            ref = v.get_field_value('reference')
+            if ref and ref[0] == '#':
+                continue
+            libpart = SubElement(libparts, 'libpart')
+            res = k.split(':')
+            if res:
+                libpart.set('lib', res[0])
+                libpart.set('part', res[1])
+            if v.alias:
+                aliases = SubElement(libpart, 'aliases')
+                for alias in v.alias:
+                    SubElement(aliases, 'alias').text = alias
+            # Description
+            desc = None
+            if v.dcm and v.dcm.desc:
+                desc = v.dcm.desc
+            else:
+                desc = v.get_field_value('ki_description')
+            if desc:
+                SubElement(libpart, 'description').text = desc
+            # Datatsheet
+            datasheet = None
+            if v.dcm and v.dcm.datasheet:
+                datasheet = v.dcm.datasheet
+            else:
+                datasheet = v.get_field_value('datasheet')
+            if datasheet:
+                SubElement(libpart, 'docs').text = datasheet
+            # Footprint filters
+            fp_list = None
+            if v.fp_list:
+                fp_list = v.fp_list
+            else:
+                fp_list = v.get_field_value('ki_fp_filters').split()
+            if fp_list:
+                fps = SubElement(libpart, 'footprints')
+                for fp in fp_list:
+                    SubElement(fps, 'fp').text = fp
+            # Fields
+            flds = SubElement(libpart, 'fields')
+            for fld in v.fields:
+                if not fld.value or fld.name.startswith('ki_'):
+                    continue
+                field = SubElement(flds, 'field')
+                field.set('name', fld.name)
+                field.text = fld.value
+            # Pins
+            if v.all_pins:
+                pins = SubElement(libpart, 'pins')
+                for pin in sorted(v.all_pins, key=lambda x: "%10s" % x.number):
+                    pn = SubElement(pins, 'pin')
+                    pn.set('num', pin.number)
+                    name = pin.name
+                    if self.netlist_version == 'E' and name == '~':
+                        name = ''
+                    pn.set('name', name)
+                    tp = pin.type
+                    if len(tp) == 1:
+                        tp = pin.type2name.get(pin.type, 'unknown')
+                    pn.set('type', tp)
+
+    def save_netlist(self, fhandle, comps, excluded=False, fitted=True, no_field=()):
+        """ This is a partial netlist in XML, only useful for BoMs """
+        root = Element("export")
+        root.set('version', self.netlist_version)
+        # Design section
+        self.save_netlist_design(root)
+        # Components
+        self.save_netlist_components(root, comps, excluded, fitted, no_field)
+        # LibParts
+        self.save_netlist_libparts(root)
+        # Libraries
+        libraries = SubElement(root, 'libraries')
+        for k, v in self.libs.items():
+            lib = SubElement(libraries, 'library')
+            lib.set('logical', k)
+            SubElement(lib, 'uri').text = v
+        # Nets
+        # TODO: May be in the future
+        SubElement(root, 'nets')
+        # Make it look nice
+        rough_string = tostring(root, encoding='utf8')
+        reparsed = minidom.parseString(rough_string)
+        fhandle.write(reparsed.toprettyxml(indent="  ", encoding='UTF-8'))
