@@ -1,18 +1,41 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020-2021 Salvador E. Tropea
-# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2022 Salvador E. Tropea
+# Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import pcbnew
 from .optionable import Optionable
 from .gs import GS
-from .misc import KICAD_VERSION_5_99, W_NOTASCII
+from .misc import W_NOTASCII
 from re import match
 from .error import (PlotError, KiPlotConfigurationError)
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
+
+
+LAYER_ORDER = ['F.Cu', 'F.Mask', 'F.SilkS', 'F.Paste', 'F.Adhes', 'F.CrtYd', 'F.Fab', 'Dwgs.User', 'Cmts.User', 'Eco1.User',
+               'Eco2.User', 'Edge.Cuts', 'Margin', 'User.1', 'User.2', 'User.3', 'User.4', 'User.5', 'User.6', 'User.7',
+               'User.8', 'User.9', 'In1.Cu', 'In2.Cu', 'In3.Cu', 'In4.Cu', 'In5.Cu', 'In6.Cu', 'In7.Cu', 'In8.Cu', 'In9.Cu',
+               'In10.Cu', 'In11.Cu', 'In12.Cu', 'In13.Cu', 'In14.Cu', 'In15.Cu', 'In16.Cu', 'In17.Cu', 'In18.Cu', 'In19.Cu',
+               'In20.Cu', 'In21.Cu', 'In22.Cu', 'In23.Cu', 'In24.Cu', 'In25.Cu', 'In26.Cu', 'In27.Cu', 'In28.Cu', 'In29.Cu',
+               'In30.Cu', 'B.Cu', 'B.Mask', 'B.SilkS', 'B.Paste', 'B.Adhes', 'B.CrtYd', 'B.Fab']
+LAYER_PRIORITY = {}
+
+
+def create_print_priority(board):
+    """ Fills LAYER_PRIORITY. This is used to sort layers for printing.
+        It is the way KiCad sorts the layers.
+        We do it as soon as we have a valid board. """
+    global LAYER_PRIORITY
+    if len(LAYER_PRIORITY) > 0:
+        return
+    LAYER_PRIORITY = {board.GetLayerID(name): c for c, name in enumerate(LAYER_ORDER)}
+
+
+def get_priority(id):
+    return LAYER_PRIORITY.get(id, 1e6)
 
 
 class Layer(Optionable):
@@ -40,28 +63,49 @@ class Layer(Optionable):
         'F.Fab': pcbnew.F_Fab,
         'B.Fab': pcbnew.B_Fab,
     }
+    # ID to default name table
+    ID_2_DEFAULT_NAME = None
     # Default names
     DEFAULT_LAYER_DESC = {
         'F.Cu': 'Front copper',
         'B.Cu': 'Bottom copper',
         'F.Adhes': 'Front adhesive (glue)',
         'B.Adhes': 'Bottom adhesive (glue)',
+        'F.Adhesive': 'Front adhesive (glue)',
+        'B.Adhesive': 'Bottom adhesive (glue)',
         'F.Paste': 'Front solder paste',
         'B.Paste': 'Bottom solder paste',
         'F.SilkS': 'Front silkscreen (artwork)',
         'B.SilkS': 'Bottom silkscreen (artwork)',
+        'F.Silkscreen': 'Front silkscreen (artwork)',
+        'B.Silkscreen': 'Bottom silkscreen (artwork)',
         'F.Mask': 'Front soldermask (negative)',
         'B.Mask': 'Bottom soldermask (negative)',
         'Dwgs.User': 'User drawings',
+        'User.Drawings': 'User drawings',
         'Cmts.User': 'User comments',
+        'User.Comments': 'User comments',
         'Eco1.User': 'For user usage 1',
         'Eco2.User': 'For user usage 2',
+        'User.Eco1': 'For user usage 1',
+        'User.Eco2': 'For user usage 2',
         'Edge.Cuts': 'Board shape',
         'Margin': 'Margin relative to edge cut',
         'F.CrtYd': 'Front courtyard area',
         'B.CrtYd': 'Bottom courtyard area',
+        'F.Courtyard': 'Front courtyard area',
+        'B.Courtyard': 'Bottom courtyard area',
         'F.Fab': 'Front documentation',
         'B.Fab': 'Bottom documentation',
+        'User.1': 'User layer 1',
+        'User.2': 'User layer 2',
+        'User.3': 'User layer 3',
+        'User.4': 'User layer 4',
+        'User.5': 'User layer 5',
+        'User.6': 'User layer 6',
+        'User.7': 'User layer 7',
+        'User.8': 'User layer 8',
+        'User.9': 'User layer 9',
     }
     KICAD6_RENAME = {
         'F.Adhes': 'F.Adhesive',
@@ -118,6 +162,11 @@ class Layer(Optionable):
             self.suffix = self.layer.replace('.', '_')
         self.clean_suffix()
 
+    @staticmethod
+    def reset():
+        Layer._pcb_layers = None
+        Layer._plot_layers = None
+
     def clean_suffix(self):
         filtered_suffix = ''.join(char for char in self.suffix if ord(char) < 128)
         if filtered_suffix != self.suffix:
@@ -143,12 +192,13 @@ class Layer(Optionable):
         self._protel_extension = 'gbr'
         return
 
-    @staticmethod
-    def solve(values):
+    @classmethod
+    def solve(cls, values):
         board = GS.board
         layer_cnt = 2
         if board:
             layer_cnt = board.GetCopperLayerCount()
+            create_print_priority(board)
         # Get the list of used layers from the board
         # Used for 'all' but also to validate the layer names
         if Layer._pcb_layers is None:
@@ -174,6 +224,8 @@ class Layer(Optionable):
                         raise PlotError("Inner layer `{}` is not valid for this board".format(layer))
                     layer.fix_protel_ext()
                     new_vals.append(layer)
+                elif isinstance(layer, int):
+                    new_vals.append(cls.create_layer(layer))
                 else:  # A string
                     ext = None
                     if layer == 'all':
@@ -187,17 +239,17 @@ class Layer(Optionable):
                     elif layer == 'user':
                         ext = Layer._get_layers(Layer._get_user())
                     elif layer in Layer._pcb_layers:
-                        ext = [Layer.create_layer(layer)]
+                        ext = [cls.create_layer(layer)]
                     # Give compatibility for the KiCad 5 default names (automagically renamed by KiCad 6)
-                    elif GS.kicad_version_n >= KICAD_VERSION_5_99 and layer in Layer.KICAD6_RENAME:  # pragma: no cover (Ki6)
-                        ext = [Layer.create_layer(Layer.KICAD6_RENAME[layer])]
+                    elif GS.ki6() and layer in Layer.KICAD6_RENAME:
+                        ext = [cls.create_layer(Layer.KICAD6_RENAME[layer])]
                     elif layer in Layer.DEFAULT_LAYER_NAMES:
-                        ext = [Layer.create_layer(layer)]
+                        ext = [cls.create_layer(layer)]
                     if ext is None:
                         raise KiPlotConfigurationError("Unknown layer spec: `{}`".format(layer))
                     new_vals.extend(ext)
             return new_vals
-        assert False, "Unimplemented layer type "+str(type(values))
+        raise AssertionError("Unimplemented layer type "+str(type(values)))
 
     @staticmethod
     def _get_copper():
@@ -218,10 +270,16 @@ class Layer(Optionable):
     @classmethod
     def create_layer(cls, name):
         layer = cls()
-        layer.layer = name
+        if isinstance(name, str):
+            layer.layer = name
+            layer._get_layer_id_from_name()
+        else:
+            layer._id = name
+            layer._is_inner = name > pcbnew.F_Cu and name < pcbnew.B_Cu
+            name = GS.board.GetLayerName(name)
+            layer.layer = name
         layer.suffix = name.replace('.', '_')
-        layer.description = Layer.DEFAULT_LAYER_DESC.get(name)
-        layer._get_layer_id_from_name()
+        layer.description = Layer.DEFAULT_LAYER_DESC.get(name, '')
         layer.fix_protel_ext()
         layer.clean_suffix()
         return layer
@@ -229,7 +287,7 @@ class Layer(Optionable):
     @staticmethod
     def _get_layers(d_layers):
         layers = []
-        for n, id in d_layers.items():
+        for n in d_layers.keys():
             layers.append(Layer.create_layer(n))
         return layers
 
@@ -265,13 +323,36 @@ class Layer(Optionable):
                 raise KiPlotConfigurationError("Unknown layer name: `{}`".format(self.layer))
         return self._id
 
+    def is_copper(self):
+        return self._id >= pcbnew.F_Cu and self._id <= pcbnew.B_Cu
+
+    def is_top(self):
+        return self._id == pcbnew.F_Cu
+
+    def is_bottom(self):
+        return self._id == pcbnew.B_Cu
+
     def __str__(self):
         if hasattr(self, '_id'):
             return "{} ({} '{}' {})".format(self.layer, self._id, self.description, self.suffix)
         return "{} ('{}' {})".format(self.layer, self.description, self.suffix)
 
+    @staticmethod
+    def id2def_name(id):
+        if GS.ki5():
+            return Layer.ID_2_DEFAULT_NAME[id]
+        return pcbnew.LayerName(id)
 
+
+# Add all the Inner layers
 for i in range(1, 30):
     name = 'In'+str(i)+'.Cu'
     Layer.DEFAULT_LAYER_NAMES[name] = pcbnew.In1_Cu+i-1
     Layer.DEFAULT_LAYER_DESC[name] = 'Inner layer '+str(i)
+if GS.ki6():
+    # Add all the User.N layers
+    for i in range(1, 10):
+        name = 'User.'+str(i)
+        Layer.DEFAULT_LAYER_NAMES[name] = pcbnew.User_1+i-1
+        Layer.DEFAULT_LAYER_DESC[name] = 'User layer '+str(i)
+Layer.ID_2_DEFAULT_NAME = {v: k for k, v in Layer.DEFAULT_LAYER_NAMES.items()}
