@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020-2021 Salvador E. Tropea
-# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2022 Salvador E. Tropea
+# Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 """ Miscellaneous definitions """
@@ -34,6 +34,10 @@ PCBDRAW_ERR = 20
 SVG_SCH_PRINT = 21
 CORRUPTED_SCH = 22
 WRONG_INSTALL = 23
+RENDER_3D_ERR = 24
+FAILED_EXECUTE = 25
+KICOST_ERROR = 26
+MISSING_WKS = 27
 error_level_to_name = ['NONE',
                        'INTERNAL_ERROR',
                        'WRONG_ARGUMENTS',
@@ -58,13 +62,21 @@ error_level_to_name = ['NONE',
                        'SVG_SCH_PRINT',
                        'CORRUPTED_SCH',
                        'WRONG_INSTALL',
+                       'RENDER_3D_ERR',
+                       'FAILED_EXECUTE',
+                       'KICOST_ERROR',
+                       'MISSING_WKS',
                        ]
 CMD_EESCHEMA_DO = 'eeschema_do'
-URL_EESCHEMA_DO = 'https://github.com/INTI-CMNB/kicad-automation-scripts'
+URL_EESCHEMA_DO = 'https://github.com/INTI-CMNB/KiAuto'
 CMD_PCBNEW_RUN_DRC = 'pcbnew_do'
 URL_PCBNEW_RUN_DRC = URL_EESCHEMA_DO
-CMD_PCBNEW_PRINT_LAYERS = 'pcbnew_do'
+CMD_PCBNEW_PRINT_LAYERS = CMD_PCBNEW_RUN_DRC
 URL_PCBNEW_PRINT_LAYERS = URL_EESCHEMA_DO
+CMD_PCBNEW_3D = CMD_PCBNEW_RUN_DRC
+URL_PCBNEW_3D = URL_EESCHEMA_DO
+CMD_PCBNEW_GENCAD = CMD_PCBNEW_RUN_DRC
+URL_PCBNEW_GENCAD = URL_EESCHEMA_DO
 CMD_KIBOM = 'KiBOM_CLI.py'
 URL_KIBOM = 'https://github.com/INTI-CMNB/KiBoM'
 CMD_IBOM = 'generate_interactive_bom.py'
@@ -72,12 +84,15 @@ URL_IBOM = 'https://github.com/INTI-CMNB/InteractiveHtmlBom'
 CMD_KICOST = 'kicost'
 URL_KICOST = 'https://github.com/INTI-CMNB/KiCost'
 KICOST_SUBMODULE = '../submodules/KiCost/src/kicost'
-KICAD2STEP = 'kicad2step'
+KICAD2STEP = 'kicad2step_do'
 PCBDRAW = 'pcbdraw'
 URL_PCBDRAW = 'https://github.com/INTI-CMNB/pcbdraw'
 EXAMPLE_CFG = 'example_template.kibot.yaml'
 AUTO_SCALE = 0
+PANDOC = 'pandoc'
 KICAD_VERSION_5_99 = 5099000
+KICAD_VERSION_6_0_0 = 6000000
+KICAD_VERSION_6_0_2 = 6000002
 
 # Internal filter names
 IFILT_MECHANICAL = '_mechanical'
@@ -124,13 +139,13 @@ DNC = {
     "fixed",
 }
 # KiCost distributors
-DISTRIBUTORS = ['digikey', 'farnell', 'mouser', 'newark', 'rs', 'arrow', 'tme', 'lcsc']
+DISTRIBUTORS = ['arrow', 'digikey', 'farnell', 'lcsc', 'mouser', 'newark', 'rs', 'tme']
 DISTRIBUTORS_F = [d+'#' for d in DISTRIBUTORS]
 # ISO ISO4217 currency codes
 # Not all, but the ones we get from the European Central Bank (march 2021)
-ISO_CURRENCIES = set(['EUR', 'USD', 'JPY', 'BGN', 'CZK', 'DKK', 'GBP', 'HUF', 'PLN', 'RON', 'SEK', 'CHF', 'ISK', 'NOK', 'HRK',
-                      'RUB', 'TRY', 'AUD', 'BRL', 'CAD', 'CNY', 'HKD', 'IDR', 'ILS', 'INR', 'KRW', 'MXN', 'MYR', 'NZD', 'PHP',
-                      'SGD', 'THB', 'ZAR'])
+ISO_CURRENCIES = {'EUR', 'USD', 'JPY', 'BGN', 'CZK', 'DKK', 'GBP', 'HUF', 'PLN', 'RON', 'SEK', 'CHF', 'ISK', 'NOK', 'HRK',
+                  'RUB', 'TRY', 'AUD', 'BRL', 'CAD', 'CNY', 'HKD', 'IDR', 'ILS', 'INR', 'KRW', 'MXN', 'MYR', 'NZD', 'PHP',
+                  'SGD', 'THB', 'ZAR'}
 
 W_VARCFG = '(W001) '
 W_VARPCB = '(W002) '
@@ -198,6 +213,47 @@ W_UNKDIST = '(W063) '
 W_UNKCUR = '(W064) '
 W_NONETLIST = '(W065) '
 W_NOKICOST = '(W066) '
+W_UNKOUT = '(W067) '
+W_NOFILTERS = '(W068) '
+W_NOVARIANTS = '(W069) '
+W_NOENDLIB = '(W070) '
+W_NEEDSPCB = '(W071) '
+W_NOGLOBALS = '(W072) '
+W_EMPTREP = '(W073) '
+W_BADCHARS = '(W074) '
+W_DATEFORMAT = '(W075) '
+W_UNKFLD = '(W076) '
+W_ALRDOWN = '(W077) '
+W_KICOSTFLD = '(W078) '
+W_MIXVARIANT = '(W079) '
+W_NOTPDF = '(W080) '
+W_NOREF = '(W081) '
+W_UNKVAR = '(W082) '
+W_WRONGEXT = '(W083) '
+W_COLORTHEME = '(W084) '
+W_WRONGCOLOR = '(W085) '
+W_WKSVERSION = '(W086) '
+W_WRONGOAR = '(W087) '
+W_ECCLASST = '(W088) '
+W_PDMASKFAIL = '(W089) '
+W_MISSTOOL = '(W090) '
+# Somehow arbitrary, the colors are real, but can be different
+PCB_MAT_COLORS = {'fr1': "937042", 'fr2': "949d70", 'fr3': "adacb4", 'fr4': "332B16", 'fr5': "6cc290"}
+PCB_FINISH_COLORS = {'hal': "8b898c", 'hasl': "8b898c", 'imag': "8b898c", 'enig': "cfb96e", 'enepig': "cfb96e",
+                     'none': "d39751", 'hal snpb': "8b898c", 'hal lead-free': "8b898c", 'hard gold': "cfb96e",
+                     'immersion silver': "8b898c", 'immersion ag': "8b898c", 'imau': "cfb96e", 'immersion gold': "cfb96e",
+                     'immersion au': "cfb96e", 'immersion tin': "8b898c", 'immersion nickel': "cfb96e", 'osp': "d39751",
+                     'ht_osp': "d39751"}
+SOLDER_COLORS = {'green': ("#285e3a", "#208b47"),
+                 'black': ("#1d1918", "#2d2522"),
+                 'blue': ("#1b1f44", "#00406a"),
+                 'red': ("#812e2a", "#be352b"),
+                 'white': ("#bdccc7", "#b7b7ad"),
+                 'yellow': ("#73823d", "#f2a756"),
+                 'purple': ("#30234a", "#451d70")}
+SILK_COLORS = {'black': "0b1013", 'white': "d5dce4"}
+# KiCad 6 uses IUs for SVGs, but KiCad 5 uses a very different scale based on inches
+KICAD5_SVG_SCALE = 116930/297002200
 
 
 class Rect(object):
@@ -228,7 +284,7 @@ def name2make(name):
 
 @contextmanager
 def hide_stderr():
-    """ Low level stderr supression, used to hide KiCad bugs. """
+    """ Low level stderr suppression, used to hide KiCad bugs. """
     newstderr = os.dup(2)
     devnull = os.open('/dev/null', os.O_WRONLY)
     os.dup2(devnull, 2)
