@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2021 Salvador E. Tropea
-# Copyright (c) 2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2021-2022 Salvador E. Tropea
+# Copyright (c) 2021-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import re
@@ -16,10 +16,11 @@ from .gs import GS
 from .kiplot import config_output, get_output_dir, run_output
 from .misc import MISSING_TOOL, WRONG_INSTALL, W_EMPTYZIP, WRONG_ARGUMENTS, INTERNAL_ERROR
 from .optionable import Optionable, BaseOptions
+from .registrable import RegOutput
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 
 
 class FilesList(Optionable):
@@ -28,7 +29,10 @@ class FilesList(Optionable):
         with document:
             self.source = '*'
             """ File names to add, wildcards allowed. Use ** for recursive match.
-                Note this pattern is applied to the output dir specified with -d comman line option """
+                By default this pattern is applied to the output dir specified with `-d` command line option.
+                See the `from_cwd` option """
+            self.from_cwd = False
+            """ Use the current working directory instead of the dir specified by `-d` """
             self.from_output = ''
             """ Collect files from the selected output.
                 When used the `source` option is ignored """
@@ -60,6 +64,10 @@ class CompressOptions(BaseOptions):
             """ [auto,stored,deflated,bzip2,lzma] Compression algorithm. Use auto to let KiBot select a suitable one """
             self.files = FilesList
             """ [list(dict)] Which files will be included """
+            self.move_files = False
+            """ Move the files to the archive. In other words: remove the files after adding them to the archive """
+            self.remove_files = None
+            """ {move_files} """
         super().__init__()
 
     def config(self, parent):
@@ -73,7 +81,7 @@ class CompressOptions(BaseOptions):
     def create_zip(self, output, files):
         extra = {}
         extra['compression'] = self.ZIP_ALGORITHMS[self.compression]
-        if sys.version_info.major > 3 or (sys.version_info.major == 3 and sys.version_info.minor >= 7):
+        if sys.version_info >= (3, 7):
             extra['compresslevel'] = 9
         with ZipFile(output, 'w', **extra) as zip:
             for fname, dest in files.items():
@@ -118,16 +126,21 @@ class CompressOptions(BaseOptions):
     def get_files(self, output, no_out_run=False):
         output_real = os.path.realpath(output)
         files = OrderedDict()
+        out_dir_cwd = os.getcwd()
+        out_dir_default = self.expand_filename_sch(GS.out_dir)
+        dirs_list = []
         for f in self.files:
             # Get the list of candidates
             files_list = None
             if f.from_output:
-                for out in GS.outputs:
-                    if out.name == f.from_output:
-                        config_output(out)
-                        files_list = out.get_targets(get_output_dir(out.dir, dry=True))
-                        break
-                if files_list is None:
+                out = RegOutput.get_output(f.from_output)
+                if out is not None:
+                    config_output(out)
+                    out_dir = get_output_dir(out.dir, out, dry=True)
+                    files_list = out.get_targets(out_dir)
+                    if out_dir not in dirs_list:
+                        dirs_list.append(out_dir)
+                else:
                     logger.error('Unknown output `{}` selected in {}'.format(f.from_output, self._parent))
                     exit(WRONG_ARGUMENTS)
                 if not no_out_run:
@@ -142,7 +155,9 @@ class CompressOptions(BaseOptions):
                                 logger.error('Unable to generate `{}` from {}'.format(file, out))
                                 exit(INTERNAL_ERROR)
             else:
-                files_list = glob.iglob(os.path.join(GS.out_dir, f.source), recursive=True)
+                out_dir = out_dir_cwd if f.from_cwd else out_dir_default
+                source = f.expand_filename_both(f.source, make_safe=False)
+                files_list = glob.iglob(os.path.join(out_dir, source), recursive=True)
             # Filter and adapt them
             for fname in filter(re.compile(f.filter).match, files_list):
                 fname_real = os.path.realpath(fname)
@@ -154,23 +169,24 @@ class CompressOptions(BaseOptions):
                 if f.dest:
                     dest = os.path.join(f.dest, os.path.basename(fname))
                 else:
-                    dest = os.path.relpath(dest, GS.out_dir)
+                    out_dir = out_dir_cwd if f.from_cwd else out_dir_default
+                    dest = os.path.relpath(dest, out_dir)
                 files[fname_real] = dest
-        return files
+        return files, dirs_list
 
     def get_targets(self, out_dir):
         return [self._parent.expand_filename(out_dir, self.output)]
 
     def get_dependencies(self):
-        output = self.get_targets(GS.out_dir)[0]
-        files = self.get_files(output, no_out_run=True)
+        output = self.get_targets(self.expand_filename_sch(GS.out_dir))[0]
+        files, _ = self.get_files(output, no_out_run=True)
         return files.keys()
 
     def run(self, output):
         # Output file name
         logger.debug('Collecting files')
         # Collect the files
-        files = self.get_files(output)
+        files, dirs_outs = self.get_files(output)
         logger.debug('Generating `{}` archive'.format(output))
         if self.format == 'ZIP':
             self.create_zip(output, files)
@@ -178,6 +194,23 @@ class CompressOptions(BaseOptions):
             self.create_tar(output, files)
         elif self.format == 'RAR':
             self.create_rar(output, files)
+        if self.move_files:
+            dirs = dirs_outs
+            for fname in files.keys():
+                if os.path.isfile(fname):
+                    os.remove(fname)
+                    logger.debug('Removing '+fname)
+                elif os.path.isdir(fname):
+                    dirs.append(fname)
+            for d in sorted(dirs, key=lambda x: len(x.split(os.sep)), reverse=True):
+                logger.debug('Removing '+d)
+                try:
+                    os.rmdir(d)
+                except OSError as e:
+                    if e.errno == 39:
+                        logger.debug(' Not empty')
+                    else:
+                        raise
 
 
 @output_class
@@ -190,6 +223,7 @@ class Compress(BaseOutput):  # noqa: F821
         with document:
             self.options = CompressOptions
             """ [dict] Options for the `compress` output """
+        self._none_related = True
 
     def get_dependencies(self):
         return self.options.get_dependencies()
