@@ -32,3 +32,15 @@ class Gerb_Drill(BaseOutput):  # noqa: F821
         with document:
             self.options = Gerb_DrillOptions
             """ [dict] Options for the `gerb_drill` output """
+
+    @staticmethod
+    def get_conf_examples(name, layers, templates):
+        gb = {}
+        outs = [gb]
+        name_u = name.upper()
+        gb['name'] = 'basic_'+name
+        gb['comment'] = 'Drill files in '+name_u+' format'
+        gb['type'] = name
+        gb['dir'] = 'Gerbers_and_Drill'
+        gb['options'] = {'map': 'gerber'}
+        return outs
