@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020-2021 Salvador E. Tropea
-# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2022 Salvador E. Tropea
+# Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import os
@@ -12,10 +12,16 @@ from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 WARNING_MIX = "Avoid using it in conjunction with with IBoM native filtering options"
 
 
+def check_tool():
+    tool = search_as_plugin(CMD_IBOM, ['InteractiveHtmlBom', 'InteractiveHtmlBom/InteractiveHtmlBom'])
+    check_script(tool, URL_IBOM)
+    return tool
+
+
 class IBoMOptions(VariantOptions):
     def __init__(self):
         with document:
@@ -41,6 +47,8 @@ class IBoMOptions(VariantOptions):
             """ [bom-only,left-right,top-bottom] Default BOM view """
             self.layer_view = 'FB'
             """ [F,FB,B] Default layer view """
+            self.no_compression = False
+            """ Disable compression of pcb data """
             self.name_format = 'ibom'
             """ Output file name format supports substitutions:
                 %f : original pcb file name without extension.
@@ -58,11 +66,20 @@ class IBoMOptions(VariantOptions):
             """ Include netlist information in output. """
             self.sort_order = 'C,R,L,D,U,Y,X,F,SW,A,~,HS,CNN,J,P,NT,MH'
             """ Default sort order for components. Must contain '~' once """
-            self.netlist_file = ''
+            self.netlist_file = None
+            """ {extra_data_file} """
+            self.extra_data_file = ''
             """ Path to netlist or xml file. You can use '%F.xml' to avoid specifying the project name.
                 Leave it blank for most uses, data will be extracted from the PCB """
             self.extra_fields = ''
-            """ Comma separated list of extra fields to pull from netlist or xml file """
+            """ Comma separated list of extra fields to pull from netlist or xml file.
+                Using 'X,Y' is a shortcut for `show_fields` and `group_fields` with values 'Value,Footprint,X,Y' """
+            self.show_fields = ''
+            """ Comma separated list of fields to show in the BOM.
+                Value and Footprint are displayed when nothing is specified """
+            self.group_fields = ''
+            """ Comma separated list of fields that components will be grouped by.
+                Value and Footprint are used when nothing is specified """
             self.normalize_field_case = False
             """ Normalize extra field name case. E.g. 'MPN' and 'mpn' will be considered the same field """
             self.blacklist = ''
@@ -103,12 +120,12 @@ class IBoMOptions(VariantOptions):
 
     def config(self, parent):
         super().config(parent)
-        self._netlist_file_guess = False
-        if not self.netlist_file and self.need_extra_fields():
-            self.netlist_file = '%F.xml'
-            self._netlist_file_guess = True
-        if self.netlist_file:
-            self.netlist_file = self.expand_filename('', self.netlist_file, 'ibom', 'xml')
+        self._extra_data_file_guess = False
+        if not self.extra_data_file and self.need_extra_fields():
+            self.extra_data_file = '%F.xml'
+            self._extra_data_file_guess = True
+        if self.extra_data_file:
+            self.extra_data_file = self.expand_filename('', self.extra_data_file, 'ibom', 'xml')
 
     def get_targets(self, out_dir):
         if self.output:
@@ -119,14 +136,13 @@ class IBoMOptions(VariantOptions):
 
     def get_dependencies(self):
         files = [GS.pcb_file]
-        if self.netlist_file and os.path.isfile(self.netlist_file):
-            files.append(self.netlist_file)
+        if self.extra_data_file and os.path.isfile(self.extra_data_file):
+            files.append(self.extra_data_file)
         return files
 
     def run(self, name):
         super().run(name)
-        tool = search_as_plugin(CMD_IBOM, ['InteractiveHtmlBom', 'InteractiveHtmlBom/InteractiveHtmlBom'])
-        check_script(tool, URL_IBOM)
+        tool = check_tool()
         logger.debug('Doing Interactive BoM')
         # Tell ibom we don't want to use the screen
         os.environ['INTERACTIVE_HTML_BOM_NO_DISPLAY'] = ''
@@ -141,14 +157,14 @@ class IBoMOptions(VariantOptions):
             output_dir = name
         cmd = [tool, GS.pcb_file, '--dest-dir', output_dir, '--no-browser', ]
         # Check if the user wants extra_fields but there is no data about them (#68)
-        if self.need_extra_fields() and not os.path.isfile(self.netlist_file):
+        if self.need_extra_fields() and not os.path.isfile(self.extra_data_file):
             logger.warning(W_NONETLIST+'iBoM needs information about user defined fields and no netlist provided')
-            if self._netlist_file_guess:
+            if self._extra_data_file_guess:
                 logger.warning(W_NONETLIST+'Create a BoM in XML format or use the `update_xml` preflight')
                 # If the name of the netlist is just a guess remove it from the options
-                self.netlist_file = ''
+                self.extra_data_file = ''
             else:
-                logger.warning(W_NONETLIST+"The file name used in `netlist_file` doesn't exist")
+                logger.warning(W_NONETLIST+"The file name used in `extra_data_file` doesn't exist")
         # Apply variants/filters
         to_remove = ','.join(self.get_not_fitted_refs())
         if self.blacklist and to_remove:
@@ -173,6 +189,8 @@ class IBoMOptions(VariantOptions):
             logger.error('Failed to create BoM, error %d', e.returncode)
             if e.output:
                 logger.debug('Output from command: '+e.output.decode())
+                if "'PCB_SHAPE' object has no attribute 'GetAngle'" in e.output.decode():
+                    logger.error("Update Interactive HTML BoM your version doesn't support KiCad 6 files")
             exit(BOM_ERROR)
         logger.debug('Output from command:\n'+cmd_output_dec+'\n')
         if output:
@@ -194,3 +212,14 @@ class IBoM(BaseOutput):  # noqa: F821
 
     def get_dependencies(self):
         return self.options.get_dependencies()
+
+    @staticmethod
+    def get_conf_examples(name, layers, templates):
+        enabled = True
+        try:
+            check_tool()
+        except SystemExit:
+            enabled = False
+        if not enabled:
+            return None
+        return BaseOutput.simple_conf_examples(name, 'Interactive HTML BoM', 'Assembly')  # noqa: F821
