@@ -16,7 +16,7 @@ from .bom.columnlist import ColumnList
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 
 CONFIG_FILENAME = 'config.kibom.ini'
 
@@ -352,7 +352,7 @@ class KiBoMOptions(BaseOptions):
                 self.conf = 'bom.ini'
         else:
             # A configuration
-            conf = os.path.abspath(os.path.join(GS.out_dir, CONFIG_FILENAME))
+            conf = os.path.abspath(os.path.join(self.expand_filename_sch(GS.out_dir), CONFIG_FILENAME))
             self.conf.save(conf)
             self.conf = conf
         self._expand_ext = self.format.lower()
