@@ -1,45 +1,44 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020-2021 Salvador E. Tropea
-# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2022 Salvador E. Tropea
+# Copyright (c) 2020-2022 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import os
-import re
 from tempfile import (NamedTemporaryFile)
 # Here we import the whole module to make monkeypatch work
 import subprocess
 import shutil
-from .misc import PCBDRAW, PCBDRAW_ERR, URL_PCBDRAW, W_AMBLIST, W_UNRETOOL, W_USESVG2, W_USEIMAGICK
+from .misc import (PCBDRAW, PCBDRAW_ERR, URL_PCBDRAW, W_AMBLIST, W_UNRETOOL, W_USESVG2, W_USEIMAGICK, PCB_MAT_COLORS,
+                   PCB_FINISH_COLORS, SOLDER_COLORS, SILK_COLORS)
 from .kiplot import check_script
-from .error import KiPlotConfigurationError
 from .gs import (GS)
 from .optionable import Optionable
 from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 SVG2PNG = 'rsvg-convert'
 CONVERT = 'convert'
+# 0.9.0 implements KiCad 6 support
+MIN_VERSION = '0.9.0'
 
 
 class PcbDrawStyle(Optionable):
-    _color_re = re.compile(r"#[A-Fa-f0-9]{6}$")
-
     def __init__(self):
         super().__init__()
         with document:
-            self.copper = "#417e5a"
+            self.copper = "#285e3a"
             """ color for the copper zones (covered by solder mask) """
-            self.board = "#4ca06c"
+            self.board = "#208b47"
             """ color for the board without copper (covered by solder mask) """
-            self.silk = "#f0f0f0"
+            self.silk = "#d5dce4"
             """ color for the silk screen """
-            self.pads = "#b5ae30"
+            self.pads = "#8b898c"
             """ color for the exposed pads (metal finish) """
             self.outline = "#000000"
             """ color for the outline """
-            self.clad = "#9c6b28"
+            self.clad = "#cabb3e"
             """ color for the PCB core (not covered by solder mask) """
             self.vcut = "#bf2600"
             """ color for the V-CUTS """
@@ -50,21 +49,38 @@ class PcbDrawStyle(Optionable):
             self.highlight_padding = 1.5
             """ [0,1000] how much the highlight extends around the component [mm] """
 
-    def validate_color(self, name):
-        color = getattr(self, name)
-        if not self._color_re.match(color):
-            raise KiPlotConfigurationError('Invalid color for `{}` use `#rrggbb` with hex digits'.format(name))
-
     def config(self, parent):
+        # Apply global defaults
+        # PCB Material
+        if GS.global_pcb_material is not None:
+            material = GS.global_pcb_material.lower()
+            for mat, color in PCB_MAT_COLORS.items():
+                if mat in material:
+                    self.clad = "#"+color
+                    break
+        # Solder mask
+        if parent.bottom:
+            name = GS.global_solder_mask_color_bottom or GS.global_solder_mask_color
+        else:
+            name = GS.global_solder_mask_color_top or GS.global_solder_mask_color
+        if name and name.lower() in SOLDER_COLORS:
+            (self.copper, self.board) = SOLDER_COLORS[name.lower()]
+        # Silk screen
+        if parent.bottom:
+            name = GS.global_silk_screen_color_bottom or GS.global_silk_screen_color
+        else:
+            name = GS.global_silk_screen_color_top or GS.global_silk_screen_color
+        if name and name.lower() in SILK_COLORS:
+            self.silk = "#"+SILK_COLORS[name.lower()]
+        # PCB Finish
+        if GS.global_pcb_finish is not None:
+            name = GS.global_pcb_finish.lower()
+            for nm, color in PCB_FINISH_COLORS.items():
+                if nm in name:
+                    self.pads = "#"+color
+                    break
         super().config(parent)
-        self.validate_color('board')
-        self.validate_color('copper')
-        self.validate_color('board')
-        self.validate_color('silk')
-        self.validate_color('pads')
-        self.validate_color('outline')
-        self.validate_color('clad')
-        self.validate_color('vcut')
+        self.validate_colors(['board', 'copper', 'board', 'silk', 'pads', 'outline', 'clad', 'vcut'])
         # Not implemented but required
         self.highlight_offset = 0
 
@@ -140,6 +156,11 @@ class PcbDrawOptions(VariantOptions):
         super().__init__()
 
     def config(self, parent):
+        # Pre-parse the bottom option
+        if 'bottom' in self._tree:
+            bot = self._tree['bottom']
+            if isinstance(bot, bool):
+                self.bottom = bot
         super().config(parent)
         # Libs
         if isinstance(self.libs, type):
@@ -171,7 +192,10 @@ class PcbDrawOptions(VariantOptions):
             self.remap = self.remap._tree
         # Style
         if isinstance(self.style, type):
-            self.style = None
+            # Apply the global defaults
+            style = PcbDrawStyle()
+            style.config(self)
+            self.style = style.to_dict()
         elif isinstance(self.style, PcbDrawStyle):
             self.style = self.style.to_dict()
         self._expand_id = 'bottom' if self.bottom else 'top'
@@ -234,7 +258,7 @@ class PcbDrawOptions(VariantOptions):
 
     def run(self, name):
         super().run(name)
-        check_script(PCBDRAW, URL_PCBDRAW, '0.6.0')
+        check_script(PCBDRAW, URL_PCBDRAW, MIN_VERSION)
         # Base command with overwrite
         cmd = [PCBDRAW]
         # Add user options
@@ -309,3 +333,34 @@ class PcbDraw(BaseOutput):  # noqa: F821
         if isinstance(self.options.style, str) and os.path.isfile(self.options.style):
             files.append(self.options.style)
         return files
+
+    @staticmethod
+    def get_conf_examples(name, layers, templates):
+        enabled = True
+        try:
+            check_script(PCBDRAW, URL_PCBDRAW, MIN_VERSION)
+        except SystemExit:
+            enabled = False
+        if not enabled:
+            return None
+        outs = []
+        for la in layers:
+            is_top = la.is_top()
+            is_bottom = la.is_bottom()
+            if not is_top and not is_bottom:
+                continue
+            id = 'top' if is_top else 'bottom'
+            for style in ['jlcpcb-green-enig', 'set-blue-enig', 'set-red-hasl']:
+                style_2 = style.replace('-', '_')
+                for fmt in ['svg', 'png', 'jpg']:
+                    gb = {}
+                    gb['name'] = 'basic_{}_{}_{}_{}'.format(name, fmt, style_2, id)
+                    gb['comment'] = 'PCB 2D render in {} format, using {} style'.format(fmt.upper(), style)
+                    gb['type'] = name
+                    gb['dir'] = os.path.join('PCB', '2D_render', style_2)
+                    ops = {'style': style, 'format': fmt}
+                    if is_bottom:
+                        ops['bottom'] = True
+                    gb['options'] = ops
+                    outs.append(gb)
+        return outs
