@@ -13,7 +13,7 @@ from .fil_base import BaseFilter
 from .macros import macros, document, variant_class  # noqa: F401
 from . import log
 
-logger = log.get_logger(__name__)
+logger = log.get_logger()
 
 
 @variant_class
@@ -29,10 +29,14 @@ class KiBoM(BaseVariant):  # noqa: F821
         self._def_dnc_filter = None
         with document:
             self.config_field = 'Config'
-            """ Name of the field used to clasify components """
+            """ Name of the field used to classify components """
             self.variant = Optionable
             """ [string|list(string)=''] Board variant(s) """
 
+    def get_variant_field(self):
+        """ Returns the name of the field used to determine if the component belongs to the variant """
+        return self.config_field
+
     def set_def_filters(self, exclude_filter, dnf_filter, dnc_filter):
         """ Filters delegated to the variant """
         self._def_exclude_filter = exclude_filter
@@ -64,12 +68,11 @@ class KiBoM(BaseVariant):  # noqa: F821
         # Config field must be lowercase
         self.config_field = self.config_field.lower()
 
-    def _variant_comp_is_fitted(self, value, config):
+    def matches_variant(self, config):
         """ Apply the variants to determine if this component will be fitted.
-            value: component value (lowercase).
-            config: content of the 'Config' field (lowercase). """
+            config: content of the 'Config' field. """
         # Variants logic
-        opts = config.split(",")
+        opts = config.lower().split(",")
         # Exclude components that match a -VARIANT
         for opt in opts:
             opt = opt.strip()
@@ -95,10 +98,9 @@ class KiBoM(BaseVariant):  # noqa: F821
             if not (c.fitted and c.included):
                 # Don't check if we already discarded it
                 continue
-            value = c.value.lower()
-            config = c.get_field_value(self.config_field).lower()
-            c.fitted = self._variant_comp_is_fitted(value, config)
+            config = c.get_field_value(self.config_field)
+            c.fitted = self.matches_variant(config)
             if not c.fitted and GS.debug_level > 2:
-                logger.debug('ref: {} value: {} config: {} variant: {} -> False'.
-                             format(c.ref, value, config, self.variant))
+                logger.debug('ref: {} config: {} variant: {} -> False'.
+                             format(c.ref, config, self.variant))
         return comps
